/* -*- c -*-
 * mycloud.h:
 * Summary: core interfaces for the mycloud library
 * Description: Provides the interfaces of the mysql library to handle
 *              cloud providers
 *
 * Copy:  Copyright (C) 2011-2012 Red Hat, Inc.
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#ifndef __MC_CLOUDLIB_H__
#define __MC_CLOUDLIB_H__

#ifdef __cplusplus
extern "C" {
#endif

/******************************************/

#define MC_UUID_BUFLEN (36)

/* 
 * initialize mycloud subsystems
 */
int mcInit(void);

/*
 * deinitialize mycloud subsystems
 */
int mcDeinit(void);

/******************************************/

/**
 * mcProvider:
 *
 * a mcProvider is a private data structure representing a cloud provider's attributes
 */
typedef struct _mcProvider mcProvider;

/**
 * mcProviderPtr:
 *
 * a mcProviderPtr is a pointer to a mcProvider struct
 */
typedef mcProvider *mcProviderPtr;

/**
 * MC_PROVIDER_USERNAME_BUFLEN:
 *
 * Macro providing the maximum length of the mcProvider username string.
 * 
 */
#define MC_PROVIDER_USERNAME_BUFLEN (256 + 1)

/**
 * MC_PROVIDER_PASSWORD_BUFLEN:
 *
 * Macro providing the maximum length of the mcProvider password string.
 * 
 */
#define MC_PROVIDER_PASSWORD_BUFLEN (256 + 1)

/**
 * MC_PROVIDER_TYPE_BUFLEN:
 *
 * Macro providing the maximum length of the mcProvider type string.
 * 
 */
#define MC_PROVIDER_TYPE_BUFLEN (256 + 1)

/**
 * MC_PROVIDER_TARGET_BUFLEN:
 *
 * Macro providing the maximum length of the mcProvider target string.
 * 
 */
#define MC_PROVIDER_TARGET_BUFLEN (256 + 1)

/**
 * mcProviderConnection:
 *
 * mcProviderConnection provides the attributes to connect to a provider
 * 
 */
typedef struct _mcProviderConnection{
  char username[MC_PROVIDER_USERNAME_BUFLEN];
  char password[MC_PROVIDER_PASSWORD_BUFLEN];
  char type[MC_PROVIDER_TARGET_BUFLEN];
  char target[MC_PROVIDER_TARGET_BUFLEN];
  mcProviderPtr provider;  /* provider we are connected to */
} mcProviderConnection;

/**
 * mcProviderConnectionPtr:
 *
 * a mcProviderConnectionPtr is a pointer to a mcProviderConnection
 */
typedef mcProviderConnection *mcProviderConnectionPtr;

/**
 * connect and populate cloud provider using the specified connection
 */
int mcProviderConnect(mcProviderConnectionPtr connection);

/**
 * disconnect from specified provider
 */
int mcProviderDisconnect(mcProviderConnectionPtr provider);


/******************************************/

/**
 * mcHardwareProfile:
 *
 * a mcHardwareProfile is a private data structure representing the sizing
 *   of an instance in a cloud
 */
typedef struct _mcHardwareProfile mcHardwareProfile;

typedef mcHardwareProfile *mcHardwareProfilePtr;

/**
 * MC_HARDWARE_PROFILE_ARCH_BUFLEN:
 *
 * Macro providing the maximum length of the mcHardwareProfileInfo arch string.
 * 
 */
#define MC_HARDWARE_PROFILE_ARCH_BUFLEN (256 + 1)

/**
 * mcHardwareProfileInfo:
 *
 * a mcHardwareProfileInfo is a data structure containing hardware profile attributes
 *
 */
typedef struct _mcHardwareProfileInfo {
  int cpus;
  int memory;
  int storage;
  char arch[MC_HARDWARE_PROFILE_ARCH_BUFLEN];
} mcHardwareProfileInfo;

typedef mcHardwareProfileInfo *mcHardwareProfileInfoPtr;

/*
 * hardware profiles on provider
 */
int mcProviderNumHardwareProfiles(mcProviderPtr provider);
int mcProviderListHardwareProfiles(mcProviderPtr provider, char **name, int maxnames);

/*
 * lookup hardware profile by index
 */
mcHardwareProfilePtr mcProviderHardwareProfileAt(mcProviderPtr provider, const int index);

/*
 * lookup hardware profile by name
 */
mcHardwareProfilePtr mcProviderHardwareProfileLookupByName(mcProviderPtr provider, const char* name);

/*
 * return info pertaining to hardware profile
 */
int mcHardwareProfileGetInfo(mcHardwareProfilePtr hwp, mcHardwareProfileInfoPtr *info);

/******************************************/

/**
 * mcDeployable:
 *
 * a mcDeployable is a private structure representing a cloud deployable
 * 
 */
typedef struct _mcDeployable mcDeployable;

/**
 * mcDeployablePtr:
 *
 * a mcDeployablePtr is a pointer to a mcDeployable
 */
typedef mcDeployable *mcDeployablePtr;

/*
 * Define and return new deployable from path to file
 */
mcDeployablePtr mcDeployableDefineFromFile(char *filePath);

/**
 * undefine defined deployable
 */
int mcDeployableUndefine(mcDeployablePtr deployable);

/******************************************/

/**
 * MC_INSTANCE_STATE_LABEL_BUFLEN:
 *
 * Macro providing the maximum length of the mcHardwareProfileInfo arch string.
 * 
 */
#define MC_INSTANCE_STATE_LABEL_BUFLEN (16 + 1)

/**
 * mcInstance:
 *
 * a mcInstance is a private data structure representing an instance created
 *   from a deployable running on a cloud provider
 */
typedef struct _mcInstance mcInstance;

/**
 * mcInstancePtr:
 *
 * a mcInstancePtr is a pointer to a mcInstance
 */
typedef mcInstance *mcInstancePtr;

typedef enum {
  MC_INSTANCE_STATE_RUNNING    = 0,
  MC_INSTANCE_STATE_TERMINATED = 1
} mcInstanceState;

typedef struct _mcInstanceInfo {
  int  state;
  char state_str[MC_INSTANCE_STATE_LABEL_BUFLEN];
} mcInstanceInfo;

typedef mcInstanceInfo *mcInstanceInfoPtr;

/*
 * Launch and return instance on cloud using deployable and hardware profile
 *
 */
mcInstancePtr mcDeployToCloud(mcDeployablePtr deployable, mcHardwareProfilePtr hwp);

/*
 * stop instance running on the cloud
 */
int mcInstanceTerminate(mcInstancePtr instance);

/**
 * deinitialize instance pointer
 */
int mcInstanceDeinit(mcInstancePtr instance);

/*
 * Instance information
 */
int mcInstanceUpdateInfo(mcInstancePtr instance, mcInstanceInfoPtr info);

/******************************************/


#ifdef __cplusplus
}
#endif

#endif
