/*
 * deltacloud.c: management of interface to deltacloud
 *
 * Copyright (C) 2011-2012 Red Hat, Inc.
 *
 *
 * See COPYING.LIB for the License of this software
 *
 * Author: Mo Morsi <mmorsi@redhat.com>
 */

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "config.h"
#include "deltacloud.h"

const char* RUBY_COMMAND            = "/usr/bin/ruby";
const char* DELTACLOUDD_COMMAND     = "/usr/bin/deltacloudd";
const char* DELTACLOUD_DEFAULT_URL = "localhost";
const char* DELTACLOUD_DEFAULT_PORT = "3001";

int dcCreateDefaultUrl(char* url){
   strncpy(url, "http://", 8);
   strncat(url, DELTACLOUD_DEFAULT_URL, MC_DELTACLOUD_HOSTNAME_BUFLEN);
   strncat(url, ":", 2);
   strncat(url, DELTACLOUD_DEFAULT_PORT,    MC_DELTACLOUD_PORT_BUFLEN);
   strncat(url, "/api", 5);
   return 0;
}

int dcCreateUrl(char* url, const char* hostname, const char* port){
   strncpy(url, "http://", 7);
   strncat(url, hostname, MC_DELTACLOUD_HOSTNAME_BUFLEN);
   strncat(url, ":", 1);
   strncat(url, port,     MC_DELTACLOUD_PORT_BUFLEN);
   strncat(url, "/api", 4);
   return 0;
}

int dcConnect(dcConnectionPtr connection, char* username, char* password){
    return deltacloud_initialize(&connection->deltacloud_connection, 
                                 connection->url, username, password);
}

int dcDisconnect(dcConnectionPtr connection){
    deltacloud_free(&connection->deltacloud_connection);
    return 0;
}

int dcServerLaunch(dcServerPtr *server, char* driver){
  pid_t pID;
  char* argv[5];
  char fullcmd[100], dc_cmd[100], *driver_opt = "-i", *argv_term = NULL;
  int status;
  mcLinkedListNodePtr current_node;
  dcServerPtr current_server;

  // see if we've already started server using the same driver
  current_node = configRetrieve()->deltacloud_servers.first;
  while(current_node != NULL){
    current_server = (dcServerPtr)current_node->value;
    if(strncmp(current_server->driver, driver, MC_DELTACLOUD_DRIVER_BUFLEN) == 0){
      *server = current_server;
      return 0;
    }
    current_node = current_node->next;
  }

  status = 0;
  pID = fork();
  if(pID == 0){       // child
    sprintf(fullcmd, "%s %s -i %s", RUBY_COMMAND, DELTACLOUDD_COMMAND, driver);
    strcpy(dc_cmd, DELTACLOUDD_COMMAND);
    argv[0] = fullcmd;
    argv[1] = dc_cmd;
    argv[2] = driver_opt;
    argv[3] = driver;
    argv[4] = argv_term;
    execv(RUBY_COMMAND, argv);

  }else if(pID < 0){  // failed to fork
    status = 1;

  }else{              // parent
    *server = malloc(sizeof(dcServer));
    (*server)->pid = pID;
    sleep(1);
    strncpy((*server)->driver, driver, MC_DELTACLOUD_DRIVER_BUFLEN);

    // add it to the list of servers
    mcLinkedListAppend(&configRetrieve()->deltacloud_servers, *server);
  }

  return status;
}

int dcServerTerm(dcServerPtr server){
  //kill(server.pid, SIGHUP);
  //sleep(3);

  //kill(server.pid, SIGINT);
  //sleep(3);

  kill(server->pid, SIGQUIT);
  sleep(1);
  
  // remove it from the list of servers
  mcLinkedListRemove(&configRetrieve()->deltacloud_servers, server);

  free(server);

  return 0;
}

BOOL dcServerIsRunning(dcServerPtr server){
  int process_status;
  pid_t spid;

  spid = waitpid(server->pid, &process_status, WNOHANG);
  return spid == 0; // -1 = error ; server->pid = not running
}

dcServerPtr dcServerListRetrieveDriver(mcLinkedListPtr servers, char* driver){
  dcServerPtr current_value;
  mcLinkedListNodePtr current;
  
  current = servers->first;
  while(current != NULL){
    current_value = (dcServerPtr)current->value;
    if(strncmp(current_value->driver, driver, MC_DELTACLOUD_DRIVER_BUFLEN) == 0)
      return current_value;
    current = current->next;
  }
  return NULL;
}

int dcRetrieveProviderDeployable(mcDeployablePtr deployable, mcProviderPtr provider, const char* id){
  struct deltacloud_image dc_img;
  int res;

  res = deltacloud_get_image_by_id(&provider->connection.deltacloud_connection,
                                   id, &dc_img);
  if(res == 0){
    strncpy(deployable->id, dc_img.id, MC_DEPLOYABLE_ID_BUFLEN);
    strncpy(deployable->name, dc_img.name, MC_DEPLOYABLE_NAME_BUFLEN);
  }

  return 0;
}

int dcCreateInstance(mcInstancePtr instance, mcProviderPtr provider){
  int res;
  struct deltacloud_create_parameter hwpp, realmp;
  struct deltacloud_create_parameter* params[2];
  struct deltacloud_instance dc_instance;
  char * instance_name;
  
  deltacloud_prepare_parameter(&hwpp, "hardware_profile", instance->hwp->name);
  deltacloud_prepare_parameter(&realmp, "realm", "us");
  params[0] = &hwpp; params[1] = &realmp;

  instance_name = NULL;
  res = deltacloud_create_instance(&provider->connection.deltacloud_connection,
                                   instance->deployable->id,
                                   *params, 2, &instance_name);
  deltacloud_get_instance_by_name(&provider->connection.deltacloud_connection, instance_name, &dc_instance);
  strncpy(instance->name, instance_name, MC_INSTANCE_NAME_BUFLEN);
  strncpy(instance->id, dc_instance.id, MC_INSTANCE_ID_BUFLEN);
  strncpy(instance->href, dc_instance.href, MC_INSTANCE_HREF_BUFLEN);
  instance->provider = provider;

  return res;
}

int dcTerminateInstance(mcInstancePtr instance){
  struct deltacloud_instance dc_instance;
  int res;

  dc_instance.id = strndup(instance->id, MC_INSTANCE_ID_BUFLEN);
  dc_instance.name = strndup(instance->name, MC_INSTANCE_NAME_BUFLEN);
  dc_instance.href = strndup(instance->href, MC_INSTANCE_HREF_BUFLEN);
  res = deltacloud_instance_destroy(&instance->provider->connection.deltacloud_connection,
                                    &dc_instance);
  return res;
}
